# GNU Enterprise Forms - GTK UI driver - Errorhandler
#
# Copyright 2001-2004 Free Software Foundation
#
# This file is part of GNU Enterprise
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: ErrorHandler.py 6148 2004-08-05 12:51:54Z johannes $

import gtk
import sys
import types

from gnue.common.apps import i18n



# -----------------------------------------------------------------------------
# Handle an error on startup of an application
# -----------------------------------------------------------------------------

def handleStartupError (errortext):
  text = "%s" % errortext
  if isinstance (text, types.StringType):
    text = unicode (text, i18n.encoding)
  print
  print '-' * 60
  print o(u_("Error: %s") % text)
  print '-' * 60

  _StartupErrorDisplay (text)

  sys.exit ()


def handleUncaughtException (extype, exvalue, traceback):
  # If we throw an exception, we don't want circular calls
  sys.excepthook = sys._orig_excepthook

  _ExceptionDisplay (extype, exvalue, traceback)

  # Restore exception handler
  sys.excepthook = handleUncaughtException


# =============================================================================
# This class provides a message box for startup errors
# =============================================================================

class _StartupErrorDisplay (gtk.MessageDialog):
  def __init__(self, errortext):
    message = u_('Unable to start GNUe Forms:\n\n%s') % errortext
    
    gtk.MessageDialog.__init__ (self, None,
                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, message)

    self.show_all ()
    self.connect ('response', lambda dialog, response: gtk.mainquit ())

    gtk.main ()


# =============================================================================
# This class provides a message box for uncaught exceptions
# =============================================================================

class _ExceptionDisplay (gtk.MessageDialog):
  def __init__(self, extype, exvalue, traceback):
    message = u_("Exception occured in GNUe Forms:\n\n"
                 "Type %(type)s\nValue %(value)s") \
              % {'type': extype,
                 'value': exvalue}
   
    gtk.MessageDialog.__init__ (self, None,
                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, message)

    self.show_all ()
    self.connect ('response', lambda dialog, response: dialog.destroy ())
