# GNU Enterprise Forms - GTK UI Driver - About Box
#
# Copyright 2001-2004 Free Software Foundation
#
# This file is part of GNU Enterprise
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: about.py 6010 2004-07-21 13:56:52Z johannes $

import gtk
import types

from gnue.common.apps import GDebug, i18n

# =============================================================================
# Class implementing an about box for GTK2
# =============================================================================

class AboutBox (gtk.Dialog):

  # ---------------------------------------------------------------------------
  # Constructor
  # ---------------------------------------------------------------------------

  def __init__ (self, params, form):
    title = params.get ('name', 'Unknown')
    gtk.Dialog.__init__ (self, title, None, gtk.DIALOG_MODAL,
                         (gtk.STOCK_OK, gtk.RESPONSE_OK))
    self.set_border_width (5)
    self.vbox.set_spacing (8)

    box = gtk.Frame (" GNUe Forms ")
    tbl = gtk.Table (2, 2)
    tbl.set_border_width (8)
    tbl.set_col_spacings (8)

    self._newLabel (u_("Version:"), tbl, 0, 1, 0, 1)
    self._newLabel (u_("Driver:"), tbl, 0, 1, 1, 2)
    self._newLabel (params ['appversion'], tbl, 1, 2, 0, 1)
    self._newLabel (params ['ui_driver'], tbl, 1, 2, 1, 2)

    box.add (tbl)
    tbl.show ()

    self.vbox.pack_start (box)
    box.show ()

    box = gtk.Frame (u_(" Form Information "))
    tbl = gtk.Table (4, 2)
    tbl.set_border_width (8)
    tbl.set_col_spacings (8)

    self._newLabel (u_("Name:"),        tbl, 0, 1, 0, 1)
    self._newLabel (u_("Version:"),     tbl, 0, 1, 1, 2)
    self._newLabel (u_("Author:"),      tbl, 0, 1, 2, 3)
    self._newLabel (u_("Description:"), tbl, 0, 1, 3, 4)

    self._newLabel (params ['name'],        tbl, 1, 2, 0, 1)
    self._newLabel (params ['formversion'], tbl, 1, 2, 1, 2)
    self._newLabel (params ['author'],      tbl, 1, 2, 2, 3)
    l = self._newLabel (params ['description'], tbl, 1, 2, 3, 4)
    l.set_line_wrap (True)

    box.add (tbl)
    tbl.show ()

    self.vbox.pack_start (box)
    box.show ()


  # ---------------------------------------------------------------------------
  # Create a new left- and top-aligned label and attach it to a given table
  # ---------------------------------------------------------------------------

  def _newLabel (self, text, table, left, right, top, bottom):
    if isinstance (text, types.StringType):
      text = unicode (text, i18n.encoding)
    result = gtk.Label (text)
    result.set_alignment (0, 0)
    table.attach (result, left, right, top, bottom)
    result.show ()
    return result

if __name__ == '__main__':
  parameters = {
    'appversion'  : 'app-version',
    'ui_driver'   : 'ui-driver-name',
    'name'        : "form-name",
    'formversion' : "form-version",
    'author'      : "form-author",
    'description' : "this is a description to the form. it "
                    "could be a much "
                    "longer text than one would excpet."
  }

  x = AboutBox (parameters, None)
  x.run ()
  x.destroy ()
